unit ultsFont;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  utsTextSuite,
  ultsTypes;

function ltsFontGetPostProcessor        (aHandle: TltsFontHandle):                                  TltsPostProcessorHandle;  stdcall;
function ltsFontGetTabWidth             (aHandle: TltsFontHandle; out aValue: Integer):             TltsErrorCode;            stdcall;
function ltsFontGetCharSpacing          (aHandle: TltsFontHandle; out aValue: Integer):             TltsErrorCode;            stdcall;
function ltsFontGetLineSpacing          (aHandle: TltsFontHandle; out aValue: Single):              TltsErrorCode;            stdcall;
function ltsFontGetMetric               (aHandle: TltsFontHandle; out aValue: TtsFontMetric):       TltsErrorCode;            stdcall;

function ltsFontGetFontname             (aHandle: TltsFontHandle):                                  PAnsiChar;                stdcall;
function ltsFontGetFacename             (aHandle: TltsFontHandle):                                  PAnsiChar;                stdcall;
function ltsFontGetStylename            (aHandle: TltsFontHandle):                                  PAnsiChar;                stdcall;
function ltsFontGetFullname             (aHandle: TltsFontHandle):                                  PAnsiChar;                stdcall;
function ltsFontGetCopyright            (aHandle: TltsFontHandle):                                  PAnsiChar;                stdcall;

function ltsFontSetPostProcessor        (aHandle: TltsFontHandle; aValue: TltsPostProcessorHandle): TltsErrorCode;            stdcall;
function ltsFontSetTabWidth             (aHandle: TltsFontHandle; aValue: Integer):                 TltsErrorCode;            stdcall;
function ltsFontSetCharSpacing          (aHandle: TltsFontHandle; aValue: Integer):                 TltsErrorCode;            stdcall;
function ltsFontSetLineSpacing          (aHandle: TltsFontHandle; aValue: Single):                  TltsErrorCode;            stdcall;

function ltsFontDestroy                 (aHandle: TltsFontHandle):                                  TltsErrorCode;            stdcall;

implementation

uses
  ultsUtils {$IFDEF DEBUG}, uutlLogger{$ENDIF};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Font//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetPostProcessor(aHandle: TltsFontHandle): TltsPostProcessorHandle; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetPostProcessor(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckFontHandle(aHandle, f)
      then result := f.PostProcessor
      else result := nil;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := nil;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetPostProcessor=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetTabWidth(aHandle: TltsFontHandle; out aValue: Integer): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetTabWidth(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then aValue := f.TabWidth
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetTabWidth=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetCharSpacing(aHandle: TltsFontHandle; out aValue: Integer): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetCharSpacing(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then aValue := f.CharSpacing
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetCharSpacing=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetLineSpacing(aHandle: TltsFontHandle; out aValue: Single): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetLineSpacing(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then aValue := f.LineSpacing
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetLineSpacing=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetMetric(aHandle: TltsFontHandle; out aValue: TtsFontMetric): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetMetric(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then aValue := f.Metric
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetMetric=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetFontname(aHandle: TltsFontHandle): PAnsiChar; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetFontname(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckFontHandle(aHandle, f)
      then result := PAnsiChar(f.Names.Fontname)
      else result := nil;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := nil;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetFontname=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetFacename(aHandle: TltsFontHandle): PAnsiChar; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetFacename(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckFontHandle(aHandle, f)
      then result := PAnsiChar(f.Names.FaceName)
      else result := nil;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := nil;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetFacename=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetStylename(aHandle: TltsFontHandle): PAnsiChar; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetStylename(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckFontHandle(aHandle, f)
      then result := PAnsiChar(f.Names.StyleName)
      else result := nil;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := nil;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetStylename=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetFullname(aHandle: TltsFontHandle): PAnsiChar; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetFullname(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckFontHandle(aHandle, f)
      then result := PAnsiChar(f.Names.FullName)
      else result := nil;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := nil;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetFullname=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontGetCopyright(aHandle: TltsFontHandle): PAnsiChar; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetCopyright(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckFontHandle(aHandle, f)
      then result := PAnsiChar(f.Names.Copyright)
      else result := nil;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := nil;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontGetCopyright=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontSetPostProcessor(aHandle: TltsFontHandle; aValue: TltsPostProcessorHandle): TltsErrorCode; stdcall;
var
  f: TtsFont;
  p: TtsPostProcessor;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetPostProcessor(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f) and CheckPostProcessorHandle(aValue, TtsPostProcessor, p)
      then f.PostProcessor := p
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetPostProcessor=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontSetTabWidth(aHandle: TltsFontHandle; aValue: Integer): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetTabWidth(Handle=%p; Value=%d)', [Pointer(aHandle), aValue]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then f.TabWidth := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetTabWidth=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontSetCharSpacing(aHandle: TltsFontHandle; aValue: Integer): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetCharSpacing(Handle=%p; Value=%d)', [Pointer(aHandle), aValue]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then f.CharSpacing := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetCharSpacing=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontSetLineSpacing(aHandle: TltsFontHandle; aValue: Single): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetLineSpacing(Handle=%p; Value=%f)', [Pointer(aHandle), aValue]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f)
      then f.LineSpacing := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontSetLineSpacing=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFontDestroy(aHandle: TltsFontHandle): TltsErrorCode; stdcall;
var
  f: TtsFont;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontDestroy(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    result := ltsErrNone;
    if CheckFontHandle(aHandle, f) then begin
      DelReference(ltsObjTypeFont, f);
      FreeAndNil(f);
    end else
      result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsFontDestroy=%d', [Integer(result)]);{$ENDIF}
end;

end.

