unit ultsGeneral;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  ultsTypes;

function ltsInitialize:       TltsErrorCode; stdcall;
function ltsGetVersion:       PAnsiChar;     stdcall;
function ltsGetLastErrorCode: TltsErrorCode; stdcall;
function ltsGetLastErrorMsg:  PAnsiChar;     stdcall;
function ltsFinalize:         TltsErrorCode; stdcall;

implementation

uses
  ultsUtils
  {$IFDEF DEBUG}, uutlLogger, utsTextSuite, ultsRenderer, ultsPostProcessor{$ENDIF}
  ;

{$IFDEF DEBUG}
  const Version = 'DebugVersion';
{$ELSE}
  {$I utsTextSuiteVersion.inc}
{$ENDIF}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//General///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsInitialize: TltsErrorCode; stdcall;
begin
  try
    {$IFDEF DEBUG}
    utlLogger.RegisterConsumer(TutlFileLogger.Create('libTextSuite.log', flmCreateNew, true));
    utlLogger.Log(nil, 'libTextSuite initialized', []);
    utlLogger.Log(nil, 'Type Sizes:', []);
    utlLogger.Log(nil, '  TtsCodePage                 %d', [SizeOf(TtsCodePage                )]);
    utlLogger.Log(nil, '  TtsFormat                   %d', [SizeOf(TtsFormat                  )]);
    utlLogger.Log(nil, '  TtsVertAlignment            %d', [SizeOf(TtsVertAlignment           )]);
    utlLogger.Log(nil, '  TtsHorzAlignment            %d', [SizeOf(TtsHorzAlignment           )]);
    utlLogger.Log(nil, '  TtsClipping                 %d', [SizeOf(TtsClipping                )]);
    utlLogger.Log(nil, '  TtsAntiAliasing             %d', [SizeOf(TtsAntiAliasing            )]);
    utlLogger.Log(nil, '  TtsBlockFlag                %d', [SizeOf(TtsBlockFlag               )]);
    utlLogger.Log(nil, '  TtsBlockFlags               %d', [SizeOf(TtsBlockFlags              )]);
    utlLogger.Log(nil, '  TtsFontStyle                %d', [SizeOf(TtsFontStyle               )]);
    utlLogger.Log(nil, '  TtsFontStyles               %d', [SizeOf(TtsFontStyles              )]);
    utlLogger.Log(nil, '  TtsColorChannel             %d', [SizeOf(TtsColorChannel            )]);
    utlLogger.Log(nil, '  TtsColorChannels            %d', [SizeOf(TtsColorChannels           )]);
    utlLogger.Log(nil, '  TtsImageMode                %d', [SizeOf(TtsImageMode               )]);
    utlLogger.Log(nil, '  TtsImageModes               %d', [SizeOf(TtsImageModes              )]);
    utlLogger.Log(nil, '  TtsColor4f                  %d', [SizeOf(TtsColor4f                 )]);
    utlLogger.Log(nil, '  PtsColor4f                  %d', [SizeOf(PtsColor4f                 )]);
    utlLogger.Log(nil, '  TtsPosition                 %d', [SizeOf(TtsPosition                )]);
    utlLogger.Log(nil, '  PtsPosition                 %d', [SizeOf(PtsPosition                )]);
    utlLogger.Log(nil, '  TtsRect                     %d', [SizeOf(TtsRect                    )]);
    utlLogger.Log(nil, '  PtsRect                     %d', [SizeOf(PtsRect                    )]);
    utlLogger.Log(nil, '  TtsVector4f                 %d', [SizeOf(TtsVector4f                )]);
    utlLogger.Log(nil, '  TtsMatrix4f                 %d', [SizeOf(TtsMatrix4f                )]);
    utlLogger.Log(nil, '  TtsGlyphMetric              %d', [SizeOf(TtsGlyphMetric             )]);
    utlLogger.Log(nil, '  TtsTextMetric               %d', [SizeOf(TtsTextMetric              )]);
    utlLogger.Log(nil, '  TtsFontMetric               %d', [SizeOf(TtsFontMetric              )]);
    utlLogger.Log(nil, '  TtsBlendValueFunc           %d', [SizeOf(TtsBlendValueFunc          )]);
    utlLogger.Log(nil, '  TtsBlendColorFunc           %d', [SizeOf(TtsBlendColorFunc          )]);
    utlLogger.Log(nil, '  TltsRendererCustomData      %d', [SizeOf(TltsRendererCustomData     )]);
    utlLogger.Log(nil, '  TltsStream                  %d', [SizeOf(TltsStream                 )]);
    utlLogger.Log(nil, '  TltsPostProcessorCustomData %d', [SizeOf(TltsPostProcessorCustomData)]);

    {$ENDIF}
    result := ltsErrNone;
    Initialize;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsInitialize=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsGetVersion: PAnsiChar; stdcall;
begin
  result := PAnsiChar(Version);
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsGetVersion=%p', [result]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsGetLastErrorCode: TltsErrorCode; stdcall;
begin
  result := LastErrorCode;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsGetLastErrorCode=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsGetLastErrorMsg: PAnsiChar; stdcall;
begin
  result := PAnsiChar(LastErrorMsg);
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsGetLastErrorMsg=%p', [Pointer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsFinalize: TltsErrorCode; stdcall;
begin
  try
    Finalize;
    result := ltsErrNone;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsGetLastErrorCode=%d', [Integer(result)]);{$ENDIF}
end;

end.

