unit ultsTextBlock;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  utsTextSuite,
  ultsTypes;

function ltsTextBlockGetRect        (aHandle: TltsTextBlockHandle; out aValue: TtsRect):            TltsErrorCode; stdcall;
function ltsTextBlockGetWidth       (aHandle: TltsTextBlockHandle; out aValue: Integer):            TltsErrorCode; stdcall;
function ltsTextBlockGetHeight      (aHandle: TltsTextBlockHandle; out aValue: Integer):            TltsErrorCode; stdcall;
function ltsTextBlockGetFlags       (aHandle: TltsTextBlockHandle; out aValue: TtsBlockFlags):      TltsErrorCode; stdcall;

function ltsTextBlockGetTop         (aHandle: TltsTextBlockHandle; out aValue: Integer):            TltsErrorCode; stdcall;
function ltsTextBlockGetLeft        (aHandle: TltsTextBlockHandle; out aValue: Integer):            TltsErrorCode; stdcall;
function ltsTextBlockGetVertAlign   (aHandle: TltsTextBlockHandle; out aValue: TtsVertAlignment):   TltsErrorCode; stdcall;
function ltsTextBlockGetHorzAlign   (aHandle: TltsTextBlockHandle; out aValue: TtsHorzAlignment):   TltsErrorCode; stdcall;
function ltsTextBlockGetClipping    (aHandle: TltsTextBlockHandle; out aValue: TtsClipping):        TltsErrorCode; stdcall;
function ltsTextBlockGetColor       (aHandle: TltsTextBlockHandle; out aValue: TtsColor4f):         TltsErrorCode; stdcall;
function ltsTextBlockGetFont        (aHandle: TltsTextBlockHandle; out aValue: TltsFontHandle):     TltsErrorCode; stdcall;

function ltsTextBlockSetTop         (aHandle: TltsTextBlockHandle; aValue: Integer):                TltsErrorCode; stdcall;
function ltsTextBlockSetLeft        (aHandle: TltsTextBlockHandle; aValue: Integer):                TltsErrorCode; stdcall;
function ltsTextBlockSetVertAlign   (aHandle: TltsTextBlockHandle; aValue: TtsVertAlignment):       TltsErrorCode; stdcall;
function ltsTextBlockSetHorzAlign   (aHandle: TltsTextBlockHandle; aValue: TtsHorzAlignment):       TltsErrorCode; stdcall;
function ltsTextBlockSetClipping    (aHandle: TltsTextBlockHandle; aValue: TtsClipping):            TltsErrorCode; stdcall;
function ltsTextBlockSetColor       (aHandle: TltsTextBlockHandle; aValue: TtsColor4f):             TltsErrorCode; stdcall;
function ltsTextBlockSetFont        (aHandle: TltsTextBlockHandle; aValue: TltsFontHandle):         TltsErrorCode; stdcall;

function ltsTextBlockGetActualHeight(aHandle: TltsTextBlockHandle):                                 Integer;       stdcall;
function ltsTextBlockGetTextWidthA  (aHandle: TltsTextBlockHandle; aText: PAnsiChar):               Integer;       stdcall;
function ltsTextBlockGetTextWidthW  (aHandle: TltsTextBlockHandle; aText: PWideChar):               Integer;       stdcall;

function ltsTextBlockTextOutA       (aHandle: TltsTextBlockHandle; aText: PAnsiChar):               TltsErrorCode; stdcall;
function ltsTextBlockTextOutW       (aHandle: TltsTextBlockHandle; aText: PWideChar):               TltsErrorCode; stdcall;

function ltsTextBlockDestroy        (aHandle: TltsTextBlockHandle):                                 TltsErrorCode; stdcall;

implementation

uses
  {$IFDEF DEBUG}uutlLogger,{$ENDIF} ultsUtils;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//TextBlock/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetRect(aHandle: TltsTextBlockHandle; out aValue: TtsRect): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetRect(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Rect
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetRect=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetWidth(aHandle: TltsTextBlockHandle; out aValue: Integer): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try             
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetWidth(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Width
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetWidth=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetHeight(aHandle: TltsTextBlockHandle; out aValue: Integer): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetHeight(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Height
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetHeight=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetFlags(aHandle: TltsTextBlockHandle; out aValue: TtsBlockFlags): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try     
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetFlags(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Flags
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetFlags=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetTop(aHandle: TltsTextBlockHandle; out aValue: Integer): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try         
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetTop(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Top
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetTop=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetLeft(aHandle: TltsTextBlockHandle; out aValue: Integer): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetLeft(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Left
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetLeft=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetVertAlign(aHandle: TltsTextBlockHandle; out aValue: TtsVertAlignment): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetVertAlign(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.VertAlign
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetVertAlign=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetHorzAlign(aHandle: TltsTextBlockHandle; out aValue: TtsHorzAlignment): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetHorzAlign(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.HorzAlign
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetHorzAlign=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetClipping(aHandle: TltsTextBlockHandle; out aValue: TtsClipping): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try      
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetClipping(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.Clipping
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetClipping=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetColor(aHandle: TltsTextBlockHandle; out aValue: TtsColor4f): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try      
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetColor(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.CurrentColor
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetColor=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetFont(aHandle: TltsTextBlockHandle; out aValue: TltsFontHandle): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetFont(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(@aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then aValue := b.CurrentFont
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetFont=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetTop(aHandle: TltsTextBlockHandle; aValue: Integer): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try      
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetTop(Handle=%p; Value=%d)', [Pointer(aHandle), aValue]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.Top := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetTop=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetLeft(aHandle: TltsTextBlockHandle; aValue: Integer): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetLeft(Handle=%p; Value=%d)', [Pointer(aHandle), aValue]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.Left := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetLeft=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetVertAlign(aHandle: TltsTextBlockHandle; aValue: TtsVertAlignment): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetVertAlign(Handle=%p; Value=%d)', [Pointer(aHandle), Integer(aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.VertAlign := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetVertAlign=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetHorzAlign(aHandle: TltsTextBlockHandle; aValue: TtsHorzAlignment): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetHorzAlign(Handle=%p; Value=%d)', [Pointer(aHandle), Integer(aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.HorzAlign := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetHorzAlign=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetClipping(aHandle: TltsTextBlockHandle; aValue: TtsClipping): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try                       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetClipping(Handle=%p; Value=%d)', [Pointer(aHandle), Integer(aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.Clipping := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetClipping=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetColor(aHandle: TltsTextBlockHandle; aValue: TtsColor4f): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try             
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetColor(Handle=%p; Color=(%f;%f;%f;%f))', [Pointer(aHandle), aValue.r, aValue.g, aValue.g, aValue.a]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.CurrentColor := aValue
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetColor=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockSetFont(aHandle: TltsTextBlockHandle; aValue: TltsFontHandle): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
  f: TtsFont;
begin
  try       
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetFont(Handle=%p; Value=%p)', [Pointer(aHandle), Pointer(aValue)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b) and CheckFontHandle(aValue, f)
      then b.CurrentFont := f
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockSetFont=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetActualHeight(aHandle: TltsTextBlockHandle): Integer; stdcall;
var
  b: TtsTextBlock;
begin
  try                  
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetActualHeight(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    if CheckTextBlockHandle(aHandle, b)
      then result := b.GetActualBlockHeight
      else result := -1;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := -1;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetActualHeight=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetTextWidthA(aHandle: TltsTextBlockHandle; aText: PAnsiChar): Integer; stdcall;
var
  b: TtsTextBlock;
begin
  try               
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetTextWidthA(Handle=%p; Text=%p)', [Pointer(aHandle), Pointer(aText)]);{$ENDIF}
    if CheckTextBlockHandle(aHandle, b)
      then result := b.GetTextWidthA(aText)
      else result := -1;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := -1;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetTextWidthA=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockGetTextWidthW(aHandle: TltsTextBlockHandle; aText: PWideChar): Integer; stdcall;
var
  b: TtsTextBlock;
begin
  try                 
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetTextWidthW(Handle=%p; Text=%p)', [Pointer(aHandle), Pointer(aText)]);{$ENDIF}
    if CheckTextBlockHandle(aHandle, b)
      then result := b.GetTextWidthW(aText)
      else result := -1;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := -1;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockGetTextWidthW=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockTextOutA(aHandle: TltsTextBlockHandle; aText: PAnsiChar): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try                 
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockTextOutA(Handle=%p; Text=%p)', [Pointer(aHandle), Pointer(aText)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.TextOutA(aText)
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockTextOutA=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockTextOutW(aHandle: TltsTextBlockHandle; aText: PWideChar): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try               
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockTextOutW(Handle=%p; Text=%p)', [Pointer(aHandle), Pointer(aText)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b)
      then b.TextOutW(aText)
      else result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockTextOutW=%d', [Integer(result)]);{$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function ltsTextBlockDestroy(aHandle: TltsTextBlockHandle): TltsErrorCode; stdcall;
var
  b: TtsTextBlock;
begin
  try              
    {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockDestroy(Handle=%p)', [Pointer(aHandle)]);{$ENDIF}
    result := ltsErrNone;
    if CheckTextBlockHandle(aHandle, b) then begin
      DelReference(ltsObjTypeTextBlock, b);
      FreeAndNil(b);
    end else
      result := LastErrorCode;
  except
    on ex: Exception do begin
      SetLastError(ex);
      result := LastErrorCode;
    end;
  end;
  {$IFDEF DEBUG}utlLogger.Log(nil, 'ltsTextBlockDestroy=%d', [Integer(result)]);{$ENDIF}
end;

end.

